@@ -1,4 +1,4 @@
-from .sorting import AlternateSort, GreedySort, LinearSort
+from .sorting import AlternateSort, FlipSort,GreedySort, LinearSort, UnidirectionalSort
 from .hatching import BaseHatcher, Hatcher, BasicIslandHatcher, InnerHatchRegion, StripeHatcher, getExposurePoints
 from .islandHatcher import Island, IslandHatcher
 from .utils import *
